@@ -7,23 +7,19 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-"""Example demonstrating a simple classifiction of a 2-D dataset"""
-
-from mvpa.suite import *
 """
-# Command above substitutes the following list
+Simple Plotting of Classifier Behavior
+======================================
+
+.. index:: plotting example
+
+This example runs a number of classifiers on a simple 2D dataset and plots the
+decision surface of each classifier.
+
+First compose some sample data -- no PyMVPA involved.
+"""
 
 import numpy as N
-import pylab as P
-
-# local imports
-from mvpa.datasets import Dataset
-from mvpa.clfs.plr import PLR
-from mvpa.clfs.ridge import RidgeReg
-from mvpa.clfs.svm import RbfNuSVMC,LinearNuSVMC
-from mvpa.clfs.knn import kNN
-"""
-
 
 # set up the labeled data
 # two skewed 2-D distributions
@@ -44,20 +40,34 @@ x2 = N.linspace(-10, 10, 100)
 x,y = N.meshgrid(x1, x2);
 feat_test = N.array((N.ravel(x), N.ravel(y)))
 
+"""Now load PyMVPA and convert the data into a proper
+:class:`~mvpa.datasets.base.Dataset`."""
+
+from mvpa.suite import *
+
 # create the pymvpa dataset from the labeled features
 patternsPos = Dataset(samples=feat_pos.T, labels=1)
 patternsNeg = Dataset(samples=feat_neg.T, labels=0)
 patterns = patternsPos + patternsNeg
 
+"""This demo utilizes a number of classifiers. The instantiation of a
+classifier involves almost no runtime costs, so it is easily possible
+compile a long list, if necessary."""
+
 # set up classifiers to try out
 clfs = {'Ridge Regression': RidgeReg(),
         'Linear SVM': LinearNuSVMC(probability=1,
                       enable_states=['probabilities']),
         'RBF SVM': RbfNuSVMC(probability=1,
                       enable_states=['probabilities']),
+        'SMLR': SMLR(lm=0.01),
         'Logistic Regression': PLR(criterion=0.00001),
         'k-Nearest-Neighbour': kNN(k=10)}
 
+"""Now we are ready to run the classifiers. The folowing loop trains
+and queries each classifier to finally generate a nice plot showing
+the decision surface of each individual classifier."""
+
 # loop over classifiers and show how they do
 fig = 0
 
@@ -94,6 +104,8 @@ for c in clfs:
     elif c == 'Logistic Regression':
         # get out the values used for the prediction
         res = N.asarray(clf.values)
+    elif c == 'SMLR':
+        res = N.asarray(clf.values[:, 1])
     else:
         # get the probabilities from the svm
         res = N.asarray([(q[1][1] - q[1][0] + 1) / 2
@@ -111,5 +123,6 @@ for c in clfs:
     # add the title
     P.title(c)
 
-# show all the cool figures
-P.show()
+if cfg.getboolean('examples', 'interactive', True):
+    # show all the cool figures
+    P.show()
