@@ -18,13 +18,21 @@ from mvpa.measures.base import Sensitivity
 from mvpa.misc.exceptions import ConvergenceError
 from mvpa.misc.state import StateVariable, Parametrized
 from mvpa.misc.param import Parameter
-from mvpa.misc import warning
+from mvpa.misc.transformers import SecondAxisMaxOfAbs
+from mvpa.base import warning, externals
 
-# Uber-fast C-version of the stepwise regression
-from mvpa.clfs.libsmlr import stepwise_regression as _cStepwiseRegression
+_DEFAULT_IMPLEMENTATION = "Python"
+if externals.exists('ctypes'):
+    # Uber-fast C-version of the stepwise regression
+    from mvpa.clfs.libsmlrc import stepwise_regression as _cStepwiseRegression
+    _DEFAULT_IMPLEMENTATION = "C"
+else:
+    _cStepwiseRegression = None
+    warning("SMLR implementation without ctypes is overwhelmingly slow."
+            " You are strongly advised to install python-ctypes")
 
 if __debug__:
-    from mvpa.misc import debug
+    from mvpa.base import debug
 
 def _label2oneofm(labels, ulabels):
     """Convert labels to one-of-M form.
@@ -43,7 +51,7 @@ def _label2oneofm(labels, ulabels):
 
 
 
-class SMLR(Classifier, Parametrized):
+class SMLR(Classifier):
     """Sparse Multinomial Logistic Regression `Classifier`.
 
     This is an implementation of the SMLR algorithm published in
@@ -52,8 +60,8 @@ class SMLR(Classifier, Parametrized):
     use this for your work.
     """
 
-    _clf_internals = [ 'smlr', 'linear', 'has_sensitivity', 'multiclass',
-                       'does_feature_selection' ] # later 'kernel-based'?
+    _clf_internals = [ 'smlr', 'linear', 'has_sensitivity', 'binary',
+                       'multiclass', 'does_feature_selection' ] # later 'kernel-based'?
 
     lm = Parameter(.1, min=1e-10, allowedtype='float',
              doc="""The penalty term lambda.  Larger values will give rise to
@@ -80,7 +88,7 @@ class SMLR(Classifier, Parametrized):
              doc="""Whether to add a bias term to allow fits to data not through
              zero""")
 
-    fit_all_weights = Parameter(False, allowedtype='bool',
+    fit_all_weights = Parameter(True, allowedtype='bool',
              doc="""Whether to fit weights for all classes or to the number of
             classes minus one.  Both should give nearly identical results, but
             if you set fit_all_weights to True it will take a little longer
@@ -88,7 +96,7 @@ class SMLR(Classifier, Parametrized):
             note that the convergence rate may be different, but convergence
             point is the same.""")
 
-    implementation = Parameter("C", allowedtype='basestring',
+    implementation = Parameter(_DEFAULT_IMPLEMENTATION, allowedtype='basestring',
              choices=["C", "Python"],
              doc="""Use C or Python as the implementation of
              stepwise_regression. C version brings significant speedup thus is
@@ -109,7 +117,12 @@ class SMLR(Classifier, Parametrized):
          # Add kernels, not just direct methods.
          """
         # init base class first
-        Parametrized.__init__(self, init_classes=[Classifier], **kwargs)
+        Classifier.__init__(self, **kwargs)
+
+        if _cStepwiseRegression is None and self.implementation == 'C':
+            warning('SMLR: C implementation is not available.'
+                    ' Using pure Python one')
+            self.implementation = 'Python'
 
         # pylint friendly initializations
         self.__ulabels = None
@@ -147,11 +160,12 @@ class SMLR(Classifier, Parametrized):
         sum2_w_diff, sum2_w_old, w_diff = 0.0, 0.0, 0.0
         p_resamp = N.ones(w.shape, dtype=N.float)
 
-        # set the random seed
-        N.random.seed(seed)
+        if seed is not None:
+            # set the random seed
+            N.random.seed(seed)
 
-        if __debug__:
-            debug("SMLR_", "random seed=%s" % seed)
+            if __debug__:
+                debug("SMLR_", "random seed=%s" % seed)
 
         # perform the optimization
         while not converged and cycles < maxiter:
@@ -430,6 +444,7 @@ class SMLR(Classifier, Parametrized):
 
     def getSensitivityAnalyzer(self, **kwargs):
         """Returns a sensitivity analyzer for SMLR."""
+        kwargs.setdefault('combiner', SecondAxisMaxOfAbs)
         return SMLRWeights(self, **kwargs)
 
 
@@ -441,45 +456,45 @@ class SMLR(Classifier, Parametrized):
 class SMLRWeights(Sensitivity):
     """`SensitivityAnalyzer` that reports the weights SMLR trained
     on a given `Dataset`.
+
+    By default SMLR provides multiple weights per feature (one per label in
+    training dataset). By default, all weights are combined into a single
+    sensitivity value. Please, see the `FeaturewiseDatasetMeasure` constructor
+    arguments how to custmize this behavior.
     """
 
     biases = StateVariable(enabled=True,
                            doc="A 1-d ndarray of biases")
 
-    def __init__(self, clf, **kwargs):
-        """Initialize the analyzer with the classifier it shall use.
+    _LEGAL_CLFS = [ SMLR ]
 
-        :Parameters:
-          clf: SMLR
-            classifier to use. Only classifiers sub-classed from
-            `SMLR` may be used.
+    def _call(self, dataset=None):
+        """Extract weights from SMLR classifier.
+
+        SMLR always has weights available, so nothing has to be computed here.
         """
-        if not isinstance(clf, SMLR):
-            raise ValueError, \
-                  "Classifier %s has to be a SMLR, but is [%s]" \
-                              % (`clf`, `type(clf)`)
+        clf = self.clf
+        weights = clf.weights
+        # XXX: MH: The following warning is inappropriate. In almost all cases
+        # SMLR will return more than one weight per feature. Even in the case of
+        # binary problem it will fit both weights by default. So unless you
+        # specify fit_all_weights=False manually this warning is always there.
+        # To much annoyance IMHO. I moved this information into the docstring,
+        # as there is no technical problem here, as FeaturewiseDatasetMeasure
+        # by default applies a combiner -- just that people should know...
+        # PLEASE ACKNOWLEDGE AND REMOVE
+        #if weights.shape[1] != 1:
+        #    warning("You are estimating sensitivity for SMLR %s with multiple"
+        #            " sensitivities available %s. Make sure that it is what you"
+        #            " intended to do" % (self, weights.shape) )
 
-        # init base classes first
-        Sensitivity.__init__(self, clf, **kwargs)
-
-
-    def _call(self, dataset):
-        """Extract weights from Linear SVM classifier.
-        """
-        if self.clf.weights.shape[1] != 1:
-            warning("You are estimating sensitivity for SMLR %s trained on %d" %
-                    (`self.clf`, self.clf.weights.shape[1]+1) +
-                    " classes. Make sure that it is what you intended to do" )
-
-        weights = self.clf.weights
-
-        if self.clf.has_bias:
-            self.biases = self.clf.biases
+        if clf.has_bias:
+            self.biases = clf.biases
 
         if __debug__:
-            debug('SVM',
+            debug('SMLR',
                   "Extracting weights for %d-class SMLR" %
-                  (self.clf.weights.shape[1]+1) +
+                  (weights.shape[1]+1) +
                   "Result: min=%f max=%f" %\
                   (N.min(weights), N.max(weights)))
 
