@@ -22,11 +22,11 @@ from mvpa.clfs.base import FeatureSelectionClassifier, SplitClassifier, \
                                  MulticlassClassifier
 from mvpa.clfs.smlr import SMLR
 from mvpa.clfs.knn import kNN
-from mvpa.clfs.gpr import GPR
+from mvpa.clfs.kernel import KernelLinear, KernelSquaredExponential
 
 # Helpers
 from mvpa.clfs.transerror import TransferError
-from mvpa.base import externals
+from mvpa.base import externals, cfg
 from mvpa.measures.anova import OneWayAnova
 from mvpa.misc.transformers import Absolute
 from mvpa.featsel.rfe import RFE
@@ -40,7 +40,8 @@ from mvpa.featsel.base import SensitivityBasedFeatureSelection
 _KNOWN_INTERNALS=[ 'knn', 'binary', 'svm', 'linear',
         'smlr', 'does_feature_selection', 'has_sensitivity',
         'multiclass', 'non-linear', 'kernel-based', 'lars',
-        'regression', 'libsvm', 'sg', 'meta', 'retrainable', 'gpr' ]
+        'regression', 'libsvm', 'sg', 'meta', 'retrainable', 'gpr',
+        'notrain2predict', 'ridge', 'blr', 'gnpp']
 
 class Warehouse(object):
     """Class to keep known instantiated classifiers
@@ -48,13 +49,25 @@ class Warehouse(object):
     Should provide easy ways to select classifiers of needed kind:
     clfs['linear', 'svm'] should return all linear SVMs
     clfs['linear', 'multiclass'] should return all linear classifiers
-     capable of doing multiclass classification
-     """
+    capable of doing multiclass classification
+    """
 
-    def __init__(self, known_tags=None):
-        self.__known_tags = Set(known_tags)
+    def __init__(self, known_tags=None, matches={}):
+        """Initialize warehouse
+
+        :Parameters:
+          known_tags : list of basestring
+            List of known tags
+          matches : dict
+            Optional dictionary of additional matches. E.g. since any
+            regression can be used as a binary classifier,
+            matches={'binary':['regression']}, would allow to provide
+            regressions also if 'binary' was requested
+            """
+        self._known_tags = Set(known_tags)
         self.__items = []
         self.__keys = Set()
+        self.__matches = matches
 
     def __getitem__(self, *args):
         if isinstance(args[0], tuple):
@@ -66,22 +79,34 @@ class Warehouse(object):
 
         # lets remove optional modifier '!'
         dargs = Set([x.lstrip('!') for x in args]).difference(
-            self.__known_tags)
+            self._known_tags)
 
         if len(dargs)>0:
             raise ValueError, "Unknown internals %s requested. Known are %s" % \
-                  (list(dargs), list(self.__known_tags))
+                  (list(dargs), list(self._known_tags))
 
         # dummy implementation for now
         result = []
+        # check every known item
         for item in self.__items:
             good = True
+            # by default each one counts
             for arg in args:
-                if (arg.startswith('!') and \
-                    (arg[1:] in item._clf_internals)) or \
-                    (not arg.startswith('!') and \
-                     (not (arg in item._clf_internals))):
-                    good = False
+                # check for rejection first
+                if arg.startswith('!'):
+                    if (arg[1:] in item._clf_internals):
+                        good = False
+                        break
+                    else:
+                        continue
+                # check for inclusion
+                found = False
+                for arg in [arg] + self.__matches.get(arg, []):
+                    if (arg in item._clf_internals):
+                        found = True
+                        break
+                good = found
+                if not good:
                     break
             if good:
                 result.append(item)
@@ -99,12 +124,12 @@ class Warehouse(object):
                 raise ValueError, "Cannot register %s " % item + \
                       "which has empty _clf_internals"
             clf_internals = Set(item._clf_internals)
-            if clf_internals.issubset(self.__known_tags):
+            if clf_internals.issubset(self._known_tags):
                 self.__items.append(item)
                 self.__keys |= clf_internals
             else:
                 raise ValueError, 'Unknown clf internal(s) %s' % \
-                      clf_internals.difference(self.__known_tags)
+                      clf_internals.difference(self._known_tags)
         return self
 
     @property
@@ -118,7 +143,8 @@ class Warehouse(object):
     def items(self):
         return self.__items
 
-clfs = Warehouse(known_tags=_KNOWN_INTERNALS)
+clfs = Warehouse(known_tags=_KNOWN_INTERNALS) # classifiers
+regrs = Warehouse(known_tags=_KNOWN_INTERNALS) # regressions
 
 # NB:
 #  - Nu-classifiers are turned off since for haxby DS default nu
@@ -128,7 +154,7 @@ clfs = Warehouse(known_tags=_KNOWN_INTERNALS)
 #
 clfs += [ SMLR(lm=0.1, implementation="C", descr="SMLR(lm=0.1)"),
           SMLR(lm=1.0, implementation="C", descr="SMLR(lm=1.0)"),
-          SMLR(lm=10.0, implementation="C", descr="SMLR(lm=10.0)"),
+          #SMLR(lm=10.0, implementation="C", descr="SMLR(lm=10.0)"),
           #SMLR(lm=100.0, implementation="C", descr="SMLR(lm=100.0)"),
           #                         SMLR(implementation="Python", descr="SMLR(Python)")
           ]
@@ -139,7 +165,8 @@ clfs += \
                             clfs['smlr'][0].descr) ]
 
 if externals.exists('libsvm'):
-    from mvpa.clfs import libsvm
+    from mvpa.clfs import libsvmc as libsvm
+    clfs._known_tags.union_update(libsvm.SVM._KNOWN_IMPLEMENTATIONS.keys())
     clfs += [libsvm.SVM(descr="libsvm.LinSVM(C=def)", probability=1),
              libsvm.SVM(
                  C=-10.0, descr="libsvm.LinSVM(C=10*def)", probability=1),
@@ -158,8 +185,17 @@ if externals.exists('libsvm'):
              #               descr='libsvm.SigmoidSVM()'),
              ]
 
+    # regressions
+    regrs._known_tags.union_update(['EPSILON_SVR', 'NU_SVR'])
+    regrs += [libsvm.SVM(svm_impl='EPSILON_SVR', descr='libsvm epsilon-SVR',
+                         regression=True),
+              libsvm.SVM(svm_impl='NU_SVR', descr='libsvm nu-SVR',
+                         regression=True)]
+
 if externals.exists('shogun'):
     from mvpa.clfs import sg
+    clfs._known_tags.union_update(sg.SVM._KNOWN_IMPLEMENTATIONS)
+
     # some classifiers are not yet ready to be used out-of-the-box in
     # PyMVPA, thus we don't populate warehouse with their instances
     bad_classifiers = [
@@ -177,7 +213,7 @@ if externals.exists('shogun'):
         # would fail with 'assertion Cache_Size > 2' if shogun < 0.6.3
         bad_classifiers.append('gnpp')
 
-    for impl in sg.svm.known_svm_impl:
+    for impl in sg.SVM._KNOWN_IMPLEMENTATIONS:
         # Uncomment the ones to disable
         if impl in bad_classifiers:
             continue
@@ -195,7 +231,16 @@ if externals.exists('shogun'):
 #           sg.SVM(descr="sg.SigmoidSVM()/%s" % impl, svm_impl=impl, kernel_type="sigmoid"),
             ]
 
-
+    for impl in ['libsvr', 'krr']:# \
+        # XXX svrlight sucks in SG -- dont' have time to figure it out
+        #+ ([], ['svrlight'])['svrlight' in sg.SVM._KNOWN_IMPLEMENTATIONS]:
+        regrs._known_tags.union_update([impl])
+        regrs += [ sg.SVM(svm_impl=impl, descr='sg.LinSVMR()/%s' % impl,
+                          regression=True),
+                   #sg.SVM(svm_impl=impl, kernel_type='RBF',
+                   #       descr='sg.RBFSVMR()/%s' % impl,
+                   #       regression=True),
+                   ]
 
 if len(clfs['svm', 'linear']) > 0:
     # if any SVM implementation is known, import default ones
@@ -240,26 +285,38 @@ clfs += \
 
 
 # GPR
-clfs += GPR(descr="GPR()")
+if externals.exists('scipy'):
+    from mvpa.clfs.gpr import GPR
+
+    clfs += GPR(kernel=KernelLinear(), descr="GPR(kernel='linear')")
+    clfs += GPR(kernel=KernelSquaredExponential(), descr="GPR(kernel='sqexp')")
+
+# BLR
+from mvpa.clfs.blr import BLR
+clfs += BLR(descr="BLR()")
 
 
 # SVM stuff
 
-if externals.exists('shogun') or externals.exists('libsvm'):
+if len(clfs['linear', 'svm']) > 0:
+
+    linearSVMC = clfs['linear', 'svm',
+                             cfg.get('svm', 'backend', default='libsvm').lower()
+                             ][0]
+
     # "Interesting" classifiers
     clfs += \
          FeatureSelectionClassifier(
-             LinearCSVMC(),
+             linearSVMC,
              SensitivityBasedFeatureSelection(
-                SMLRWeights(SMLR(lm=1.0, implementation="C")),
+                SMLRWeights(SMLR(lm=0.1, implementation="C")),
                 RangeElementSelector(mode='select')),
-             descr="LinSVM on SMLR(lm=1) non-0")
+             descr="LinSVM on SMLR(lm=0.1) non-0")
 
 
-    # "Interesting" classifiers
     clfs += \
         FeatureSelectionClassifier(
-            LinearCSVMC(),
+            linearSVMC,
             SensitivityBasedFeatureSelection(
                 SMLRWeights(SMLR(lm=1.0, implementation="C")),
                 RangeElementSelector(mode='select')),
@@ -277,7 +334,7 @@ if externals.exists('shogun') or externals.exists('libsvm'):
 
     clfs += \
         FeatureSelectionClassifier(
-            LinearCSVMC(),
+            linearSVMC,
             SensitivityBasedFeatureSelection(
                OneWayAnova(),
                FractionTailSelector(0.05, mode='select', tail='upper')),
@@ -285,27 +342,25 @@ if externals.exists('shogun') or externals.exists('libsvm'):
 
     clfs += \
         FeatureSelectionClassifier(
-            LinearCSVMC(),
+            linearSVMC,
             SensitivityBasedFeatureSelection(
                OneWayAnova(),
                FixedNElementTailSelector(50, mode='select', tail='upper')),
             descr="LinSVM on 50(ANOVA)")
 
-    sample_linear_svm = clfs['linear', 'svm'][0]
-
     clfs += \
         FeatureSelectionClassifier(
-            sample_linear_svm,
+            linearSVMC,
             SensitivityBasedFeatureSelection(
-               sample_linear_svm.getSensitivityAnalyzer(transformer=Absolute),
+               linearSVMC.getSensitivityAnalyzer(transformer=Absolute),
                FractionTailSelector(0.05, mode='select', tail='upper')),
             descr="LinSVM on 5%(SVM)")
 
     clfs += \
         FeatureSelectionClassifier(
-            sample_linear_svm,
+            linearSVMC,
             SensitivityBasedFeatureSelection(
-               sample_linear_svm.getSensitivityAnalyzer(transformer=Absolute),
+               linearSVMC.getSensitivityAnalyzer(transformer=Absolute),
                FixedNElementTailSelector(50, mode='select', tail='upper')),
             descr="LinSVM on 50(SVM)")
 
@@ -314,7 +369,7 @@ if externals.exists('shogun') or externals.exists('libsvm'):
     # other terms leave-1-out error on the same dataset
     # Has to be bound outside of the RFE definition since both analyzer and
     # error should use the same instance.
-    rfesvm_split = SplitClassifier(LinearCSVMC())#clfs['LinearSVMC'][0])
+    rfesvm_split = SplitClassifier(linearSVMC)#clfs['LinearSVMC'][0])
 
     # "Almost" classical RFE. If this works it would differ only that
     # our transfer_error is based on internal splitting and classifier used
@@ -330,7 +385,7 @@ if externals.exists('shogun') or externals.exists('libsvm'):
     #        sensitivity_analyzer=rfesvm_split.getSensitivityAnalyzer(),
     #        transfer_error=ConfusionBasedError(
     #           rfesvm_split,
-    #           confusion_state="training_confusions"),
+    #           confusion_state="confusion"),
     #           # and whose internal error we use
     #        feature_selector=FractionTailSelector(
     #                           0.2, mode='discard', tail='lower'),
@@ -347,7 +402,7 @@ if externals.exists('shogun') or externals.exists('libsvm'):
     #        sensitivity_analyzer=rfesvm_split.getSensitivityAnalyzer(),
     #        transfer_error=ConfusionBasedError(
     #           rfesvm_split,
-    #           confusion_state="training_confusions"),
+    #           confusion_state="confusion"),
     #           # and whose internal error we use
     #        feature_selector=FractionTailSelector(
     #                           0.2, mode='discard', tail='lower'),
@@ -394,4 +449,3 @@ if externals.exists('shogun') or externals.exists('libsvm'):
     #        # update sensitivity at each step
     #   splitter = OddEvenSplitter(),
     #   descr='LinSVM+RFE(OddEven)')
-
