@@ -9,24 +9,29 @@
 """Base class for data measures: algorithms that quantify properties of
 datasets.
 
-Besides the `DatasetMeasure` base class this module also provides the (abstract)
-`FeaturewiseDatasetMeasure` class. The difference between a general measure and
-the output of the `FeaturewiseDatasetMeasure` is that the latter returns a 1d map
-(one value per feature in the dataset). In contrast there are no restrictions
-on the returned value of `DatasetMeasure` except for that it has to be in some
-iterable container.
+Besides the `DatasetMeasure` base class this module also provides the
+(abstract) `FeaturewiseDatasetMeasure` class. The difference between a general
+measure and the output of the `FeaturewiseDatasetMeasure` is that the latter
+returns a 1d map (one value per feature in the dataset). In contrast there are
+no restrictions on the returned value of `DatasetMeasure` except for that it
+has to be in some iterable container.
+
 """
 
 __docformat__ = 'restructuredtext'
 
-import copy
+import numpy as N
+import mvpa.misc.copy as copy
 
 from mvpa.misc.state import StateVariable, Stateful
+from mvpa.misc.args import group_kwargs
 from mvpa.misc.transformers import FirstAxisMean, SecondAxisSumOfAbs
 from mvpa.base.dochelpers import enhancedDocString
+from mvpa.base import externals
+from mvpa.clfs.stats import autoNullDist
 
 if __debug__:
-    from mvpa.misc import debug
+    from mvpa.base import debug
 
 
 class DatasetMeasure(Stateful):
@@ -57,8 +62,11 @@ class DatasetMeasure(Stateful):
             "transformation algorithm")
     null_prob = StateVariable(enabled=True)
     """Stores the probability of a measure under the NULL hypothesis"""
+    null_t = StateVariable(enabled=False)
+    """Stores the t-score corresponding to null_prob under assumption
+    of Normal distribution"""
 
-    def __init__(self, transformer=None, null_dist=None, *args, **kwargs):
+    def __init__(self, transformer=None, null_dist=None, **kwargs):
         """Does nothing special.
 
         :Parameter:
@@ -73,7 +81,11 @@ class DatasetMeasure(Stateful):
         self.__transformer = transformer
         """Functor to be called in return statement of all subclass __call__()
         methods."""
-        self.__null_dist = null_dist
+        null_dist_ = autoNullDist(null_dist)
+        if __debug__:
+            debug('SA', 'Assigning null_dist %s whenever original given was %s'
+                  % (null_dist_, null_dist))
+        self.__null_dist = null_dist_
 
 
     __doc__ = enhancedDocString('DatasetMeasure', locals(), Stateful)
@@ -90,9 +102,6 @@ class DatasetMeasure(Stateful):
         """
         result = self._call(dataset)
         result = self._postcall(dataset, result)
-        self.raw_result = result
-        if not self.__transformer is None:
-            result = self.__transformer(result)
         return result
 
 
@@ -110,8 +119,18 @@ class DatasetMeasure(Stateful):
     def _postcall(self, dataset, result):
         """Some postprocessing on the result
         """
+        self.raw_result = result
+        if not self.__transformer is None:
+            if __debug__:
+                debug("SA_", "Applying transformer %s" % self.__transformer)
+            result = self.__transformer(result)
+
         # estimate the NULL distribution when functor is given
         if not self.__null_dist is None:
+            if __debug__:
+                debug("SA_", "Estimating NULL distribution using %s"
+                      % self.__null_dist)
+
             # we need a matching datameasure instance, but we have to disable
             # the estimation of the null distribution in that child to prevent
             # infinite looping.
@@ -119,18 +138,58 @@ class DatasetMeasure(Stateful):
             measure.__null_dist = None
             self.__null_dist.fit(measure, dataset)
 
-            # get probability of result under NULL hypothesis if available
-            self.null_prob = self.__null_dist.cdf(result)
+            if self.states.isEnabled('null_t'):
+                # get probability under NULL hyp, but also request
+                # either it belong to the right tail
+                null_prob, null_right_tail = \
+                           self.__null_dist.p(result, return_tails=True)
+                self.null_prob = null_prob
+
+                externals.exists('scipy', raiseException=True)
+                from scipy.stats import norm
+
+                # TODO: following logic should appear in NullDist,
+                #       not here
+                tail = self.null_dist.tail
+                if tail == 'left':
+                    acdf = N.abs(null_prob)
+                elif tail == 'right':
+                    acdf = 1.0 - N.abs(null_prob)
+                elif tail in ['any', 'both']:
+                    acdf = 1.0 - N.clip(N.abs(null_prob), 0, 0.5)
+                else:
+                    raise RuntimeError, 'Unhandled tail %s' % tail
+                # We need to clip to avoid non-informative inf's ;-)
+                # that happens due to lack of precision in mantissa
+                # which is 11 bits in double. We could clip values
+                # around 0 at as low as 1e-100 (correspond to z~=21),
+                # but for consistency lets clip at 1e-16 which leads
+                # to distinguishable value around p=1 and max z=8.2.
+                # Should be sufficient range of z-values ;-)
+                clip = 1e-16
+                null_t = norm.ppf(N.clip(acdf, clip, 1.0 - clip))
+                null_t[~null_right_tail] *= -1.0 # revert sign for negatives
+                self.null_t = null_t                 # store
+            else:
+                # get probability of result under NULL hypothesis if available
+                # and don't request tail information
+                self.null_prob = self.__null_dist.p(result)
 
         return result
 
 
-    def __str__(self):
-        return "%s(transformer=%s, enable_states=%s)" % \
-               (self.__class__.__name__, self.__transformer,
-                str(self.states.enabled))
+    def __repr__(self, prefixes=[]):
+        prefixes = prefixes[:]
+        if self.__transformer is not None:
+            prefixes.append("transformer=%s" % self.__transformer)
+        if self.__null_dist is not None:
+            prefixes.append("null_dist=%s" % self.__null_dist)
+        return super(DatasetMeasure, self).__repr__(prefixes=prefixes)
 
 
+    @property
+    def null_dist(self): return self.__null_dist
+
 
 class FeaturewiseDatasetMeasure(DatasetMeasure):
     """A per-feature-measure computed from a `Dataset` (base class).
@@ -142,19 +201,39 @@ class FeaturewiseDatasetMeasure(DatasetMeasure):
         doc="Stores basic sensitivities if the sensitivity " +
             "relies on combining multiple ones")
 
-    def __init__(self, combiner=SecondAxisSumOfAbs, *args, **kwargs):
+    # XXX should we may be default to combiner=None to avoid
+    # unexpected results? Also rethink if we need combiner here at
+    # all... May be combiners should be 'adjoint' with transformer
+    # YYY in comparison to CombinedSensitivityAnalyzer here default
+    #     value for combiner is worse than anywhere. From now on,
+    #     default combiners should be provided "in place", ie
+    #     in SMLR it makes sense to have SecondAxisMaxOfAbs,
+    #     in SVM (pair-wise) only for not-binary should be
+    #     SecondAxisSumOfAbs, though could be Max as well... uff
+    #   YOH: started to do so, but still have issues... thus
+    #        reverting back for now
+    def __init__(self, combiner=SecondAxisSumOfAbs, **kwargs): # SecondAxisSumOfAbs
         """Initialize
 
         :Parameters:
           combiner : Functor
-            If _call returned value is 2d -- combines along 2nd
-            dimension as well as sets base_sensitivities
-            TODO change combiner's default
+            The combiner is only applied if the computed featurewise dataset
+            measure is more than one-dimensional. This is different from a
+            `transformer`, which is always applied. By default, the sum of
+            absolute values along the second axis is computed.
         """
-        DatasetMeasure.__init__(self, *(args), **(kwargs))
+        DatasetMeasure.__init__(self, **kwargs)
 
         self.__combiner = combiner
 
+    def __repr__(self, prefixes=None):
+        if prefixes is None:
+            prefixes = []
+        if self.__combiner != SecondAxisSumOfAbs:
+            prefixes.append("combiner=%s" % self.__combiner)
+        return \
+            super(FeaturewiseDatasetMeasure, self).__repr__(prefixes=prefixes)
+
 
     def _call(self, dataset):
         """Computes a per-feature-measure on a given `Dataset`.
@@ -173,8 +252,19 @@ class FeaturewiseDatasetMeasure(DatasetMeasure):
          CombinedSensitivityAnalyzer, thus this one might make use of
          CombinedSensitivityAnalyzer yoh thinks, and here
          base_sensitivities doesn't sound appropriate.
+         MH: There is indeed some overlap, but also significant differences.
+             This one operates on a single sensana and combines over second
+             axis, CombinedFeaturewiseDatasetMeasure uses first axis.
+             Additionally, 'Sensitivity' base class is
+             FeaturewiseDatasetMeasures which would have to be changed to
+             CombinedFeaturewiseDatasetMeasure to deal with stuff like
+             SMLRWeights that return multiple sensitivity values by default.
+             Not sure if unification of both (and/or removal of functionality
+             here does not lead to an overall more complicated situation,
+             without any real gain -- after all this one works ;-)
         """
-        if len(result.shape)>1:
+        rsshape = result.squeeze().shape
+        if len(result.squeeze().shape)>1:
             n_base = result.shape[1]
             """Number of base sensitivities"""
             if self.states.isEnabled('base_sensitivities'):
@@ -185,8 +275,9 @@ class FeaturewiseDatasetMeasure(DatasetMeasure):
                     biases = self.biases
                     if len(self.biases) != n_base:
                         raise ValueError, \
-                            "Number of biases %d is different" % len(self.biases)\
-                            + " from number of base sensitivities %d" % n_base
+                          "Number of biases %d is " % len(self.biases) \
+                          + "different from number of base sensitivities" \
+                          + "%d" % n_base
                 for i in xrange(n_base):
                     if not biases is None:
                         bias = biases[i]
@@ -199,7 +290,13 @@ class FeaturewiseDatasetMeasure(DatasetMeasure):
 
             # After we stored each sensitivity separately,
             # we can apply combiner
-            result = self.__combiner(result)
+            if self.__combiner is not None:
+                result = self.__combiner(result)
+        else:
+            # remove bogus dimensions
+            # XXX we might need to come up with smth better. May be some naive
+            # combiner? :-)
+            result = result.squeeze()
 
         # call base class postcall
         result = DatasetMeasure._postcall(self, dataset, result)
@@ -224,7 +321,7 @@ class StaticDatasetMeasure(DatasetMeasure):
           bias
              optionally available bias
         """
-        DatasetMeasure.__init__(self, *(args), **(kwargs))
+        DatasetMeasure.__init__(self, *args, **kwargs)
         if measure is None:
             raise ValueError, "Sensitivity measure has to be provided"
         self.__measure = measure
@@ -245,13 +342,17 @@ class StaticDatasetMeasure(DatasetMeasure):
 
 class Sensitivity(FeaturewiseDatasetMeasure):
 
+    _LEGAL_CLFS = []
+    """If Sensitivity is classifier specific, classes of classifiers
+    should be listed in the list
+    """
+
     def __init__(self, clf, force_training=True, **kwargs):
         """Initialize the analyzer with the classifier it shall use.
 
         :Parameters:
-          clf : Classifier
-            classifier to use. Only classifiers sub-classed from
-            `LinearSVM` may be used.
+          clf : :class:`Classifier`
+            classifier to use.
           force_training : Bool
             if classifier was already trained -- do not retrain
         """
@@ -259,29 +360,53 @@ class Sensitivity(FeaturewiseDatasetMeasure):
         """Does nothing special."""
         FeaturewiseDatasetMeasure.__init__(self, **kwargs)
 
+        _LEGAL_CLFS = self._LEGAL_CLFS
+        if len(_LEGAL_CLFS) > 0:
+            found = False
+            for clf_class in _LEGAL_CLFS:
+                if isinstance(clf, clf_class):
+                    found = True
+                    break
+            if not found:
+                raise ValueError, \
+                  "Classifier %s has to be of allowed class (%s), but is %s" \
+                              % (clf, _LEGAL_CLFS, `type(clf)`)
+
         self.__clf = clf
         """Classifier used to computed sensitivity"""
 
         self._force_training = force_training
         """Either to force it to train"""
 
-    def __repr__(self):
-        return \
-            "<%s on %s, force_training=%s>" % \
-               (str(self), `self.__clf`, str(self._force_training))
+    def __repr__(self, prefixes=None):
+        if prefixes is None:
+            prefixes = []
+        prefixes.append("clf=%s" % repr(self.clf))
+        if not self._force_training:
+            prefixes.append("force_training=%s" % self._force_training)
+        return super(Sensitivity, self).__repr__(prefixes=prefixes)
 
 
-    def __call__(self, dataset):
+    def __call__(self, dataset=None):
         """Train classifier on `dataset` and then compute actual sensitivity.
+
+        If the classifier is already trained it is possible to extract the
+        sensitivities without passing a dataset.
         """
-        if not self.clf.trained or self._force_training:
+        # local bindings
+        clf = self.__clf
+        if not clf.trained or self._force_training:
+            if dataset is None:
+                raise ValueError, \
+                      "Training classifier to compute sensitivities requires " \
+                      "a dataset."
             if __debug__:
                 debug("SA", "Training classifier %s %s" %
-                      (`self.clf`,
+                      (`clf`,
                        {False: "since it wasn't yet trained",
                         True:  "although it was trained previousely"}
-                       [self.clf.trained]))
-            self.clf.train(dataset)
+                       [clf.trained]))
+            clf.train(dataset)
 
         return FeaturewiseDatasetMeasure.__call__(self, dataset)
 
@@ -289,6 +414,14 @@ class Sensitivity(FeaturewiseDatasetMeasure):
     def _setClassifier(self, clf):
         self.__clf = clf
 
+
+    @property
+    def feature_ids(self):
+        """Return feature_ids used by the underlying classifier
+        """
+        return self.__clf._getFeatureIds()
+
+
     clf = property(fget=lambda self:self.__clf,
                    fset=_setClassifier)
 
@@ -300,10 +433,21 @@ class CombinedFeaturewiseDatasetMeasure(FeaturewiseDatasetMeasure):
     sensitivities = StateVariable(enabled=False,
         doc="Sensitivities produced by each classifier")
 
+    # XXX think again about combiners... now we have it in here and as
+    #     well as in the parent -- FeaturewiseDatasetMeasure...
+    # YYY because we don't use parent's _call. Needs RF
     def __init__(self, analyzers=None,
-                 combiner=FirstAxisMean,
+                 combiner=None, #FirstAxisMean,
                  **kwargs):
-        if analyzers == None:
+        """Initialize CombinedFeaturewiseDatasetMeasure
+
+        :Parameters:
+          analyzers : list or None
+            List of analyzers to be used. There is no logic to populate
+            such a list in __call__, so it must be either provided to
+            the constructor or assigned to .analyzers prior calling
+        """
+        if analyzers is None:
             analyzers = []
 
         FeaturewiseDatasetMeasure.__init__(self, **kwargs)
@@ -314,24 +458,27 @@ class CombinedFeaturewiseDatasetMeasure(FeaturewiseDatasetMeasure):
         """Which functor to use to combine all sensitivities"""
 
 
-
     def _call(self, dataset):
         sensitivities = []
-        ind = 0
-        for analyzer in self.__analyzers:
+        for ind,analyzer in enumerate(self.__analyzers):
             if __debug__:
                 debug("SA", "Computing sensitivity for SA#%d:%s" %
                       (ind, analyzer))
             sensitivity = analyzer(dataset)
             sensitivities.append(sensitivity)
-            ind += 1
 
         self.sensitivities = sensitivities
         if __debug__:
-            debug("SA", "Returning combined using %s sensitivity across %d items" %
-                  (`self.__combiner`, len(sensitivities)))
+            debug("SA",
+                  "Returning combined using %s sensitivity across %d items" %
+                  (self.__combiner, len(sensitivities)))
 
-        return self.__combiner(sensitivities)
+        if self.__combiner is not None:
+            sensitivities = self.__combiner(sensitivities)
+        else:
+            # assure that we have an ndarray on output
+            sensitivities = N.asarray(sensitivities)
+        return sensitivities
 
 
     def _setAnalyzers(self, analyzers):
@@ -349,20 +496,37 @@ class CombinedFeaturewiseDatasetMeasure(FeaturewiseDatasetMeasure):
 class BoostedClassifierSensitivityAnalyzer(Sensitivity):
     """Set sensitivity analyzers to be merged into a single output"""
 
+
+    # XXX we might like to pass parameters also for combined_analyzer
+    @group_kwargs(prefixes=['slave_'], assign=True)
     def __init__(self,
                  clf,
                  analyzer=None,
                  combined_analyzer=None,
+                 slave_kwargs={},
                  **kwargs):
         """Initialize Sensitivity Analyzer for `BoostedClassifier`
+
+        :Parameters:
+          clf : `BoostedClassifier`
+            Classifier to be used
+          analyzer : analyzer
+            Is used to populate combined_analyzer
+          slave_*
+            Arguments to pass to created analyzer if analyzer is None
         """
         Sensitivity.__init__(self, clf, **kwargs)
         if combined_analyzer is None:
+            # sanitarize kwargs
+            kwargs.pop('force_training', None)
             combined_analyzer = CombinedFeaturewiseDatasetMeasure(**kwargs)
         self.__combined_analyzer = combined_analyzer
         """Combined analyzer to use"""
 
-        self.__analyzer = None
+        if analyzer is not None and len(self._slave_kwargs):
+            raise ValueError, \
+                  "Provide either analyzer of slave_* arguments, not both"
+        self.__analyzer = analyzer
         """Analyzer to use for basic classifiers within boosted classifier"""
 
 
@@ -371,7 +535,7 @@ class BoostedClassifierSensitivityAnalyzer(Sensitivity):
         # create analyzers
         for clf in self.clf.clfs:
             if self.__analyzer is None:
-                analyzer = clf.getSensitivityAnalyzer()
+                analyzer = clf.getSensitivityAnalyzer(**(self._slave_kwargs))
                 if analyzer is None:
                     raise ValueError, \
                           "Wasn't able to figure basic analyzer for clf %s" % \
@@ -392,7 +556,11 @@ class BoostedClassifierSensitivityAnalyzer(Sensitivity):
 
         self.__combined_analyzer.analyzers = analyzers
 
-        return self.__combined_analyzer(dataset)
+        # XXX not sure if we don't want to call directly ._call(dataset) to avoid
+        # double application of transformers/combiners, after all we are just
+        # 'proxying' here to combined_analyzer...
+        # YOH: decided -- lets call ._call
+        return self.__combined_analyzer._call(dataset)
 
     combined_analyzer = property(fget=lambda x:x.__combined_analyzer)
 
@@ -400,6 +568,7 @@ class BoostedClassifierSensitivityAnalyzer(Sensitivity):
 class ProxyClassifierSensitivityAnalyzer(Sensitivity):
     """Set sensitivity analyzer output just to pass through"""
 
+    @group_kwargs(prefixes=['slave_'], assign=True)
     def __init__(self,
                  clf,
                  analyzer=None,
@@ -408,29 +577,54 @@ class ProxyClassifierSensitivityAnalyzer(Sensitivity):
         """
         Sensitivity.__init__(self, clf, **kwargs)
 
-        self.__analyzer = None
+        if analyzer is not None and len(self._slave_kwargs):
+            raise ValueError, \
+                  "Provide either analyzer of slave_* arguments, not both"
+
+        self.__analyzer = analyzer
         """Analyzer to use for basic classifiers within boosted classifier"""
 
 
     def _call(self, dataset):
-        if self.__analyzer is None:
-            self.__analyzer = self.clf.clf.getSensitivityAnalyzer()
-            if self.__analyzer is None:
+        # OPT: local bindings
+        clfclf = self.clf.clf
+        analyzer = self.__analyzer
+
+        if analyzer is None:
+            analyzer = clfclf.getSensitivityAnalyzer(
+                **(self._slave_kwargs))
+            if analyzer is None:
                 raise ValueError, \
                       "Wasn't able to figure basic analyzer for clf %s" % \
-                      `self.clf.clf`
+                      `clfclf`
             if __debug__:
                 debug("SA", "Selected analyzer %s for clf %s" % \
-                      (`self.__analyzer`, `self.clf.clf`))
+                      (analyzer, clfclf))
+            # bind to the instance finally
+            self.__analyzer = analyzer
 
         # TODO "remove" unnecessary things below on each call...
         # assign corresponding classifier
-        self.__analyzer.clf = self.clf.clf
+        analyzer.clf = clfclf
 
         # if clf was trained already - don't train again
-        if self.clf.clf.trained:
-            self.__analyzer._force_training = False
+        if clfclf.trained:
+            analyzer._force_training = False
 
-        return self.__analyzer._call(dataset)
+        return analyzer._call(dataset)
 
     analyzer = property(fget=lambda x:x.__analyzer)
+
+
+class MappedClassifierSensitivityAnalyzer(ProxyClassifierSensitivityAnalyzer):
+    """Set sensitivity analyzer output be reverse mapped using mapper of the
+    slave classifier"""
+
+    def _call(self, dataset):
+        sens = super(MappedClassifierSensitivityAnalyzer, self)._call(dataset)
+        # So we have here the case that some sensitivities are given
+        #  as nfeatures x nclasses, thus we need to take .T for the
+        #  mapper and revert back afterwards
+        # devguide's TODO lists this point to 'disguss'
+        sens_mapped = self.clf.maskclf.mapper.reverse(sens.T)
+        return sens_mapped.T
