@@ -12,14 +12,15 @@ import unittest
 from mvpa.misc.cmdline import *
 
 if __debug__:
-    from mvpa.misc import debug
+    from mvpa.base import debug
 
 class CmdlineHelpersTest(unittest.TestCase):
 
     def testBasic(self):
         """Test if we are not missing basic parts"""
         globals_ = globals()
-        for member in  ['verboseCallback', 'parser', 'optHelp', 'optVerbose']:
+        for member in  [#'_verboseCallback',
+                        'parser', 'opt', 'opts']:
             self.failUnless(globals_.has_key(member),
                 msg="We must have imported %s from mvpa.misc.cmdline!" % member)
 
