@@ -8,19 +8,17 @@
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 """Unit tests for PyMVPA least angle regression (LARS) classifier"""
 
-import unittest
-from mvpa.datasets import Dataset
 from mvpa.clfs.lars import LARS
-import numpy as N
 from scipy.stats import pearsonr
-from mvpa.misc.data_generators import dumbFeatureBinaryDataset
+from tests_warehouse import *
+from mvpa.misc.data_generators import normalFeatureDataset
 
 class LARSTests(unittest.TestCase):
 
     def testLARS(self):
         # not the perfect dataset with which to test, but
         # it will do for now.
-        data = dumbFeatureBinaryDataset()
+        data = datasets['dumb2']
 
         clf = LARS(regression=True)
 
@@ -33,7 +31,7 @@ class LARSTests(unittest.TestCase):
         self.failUnless(cor[0] > .8)
 
     def testLARSState(self):
-        data = dumbFeatureBinaryDataset()
+        data = datasets['dumb2']
 
         clf = LARS()
 
@@ -46,6 +44,20 @@ class LARSTests(unittest.TestCase):
         self.failUnless((p == clf.predictions).all())
 
 
+    def testLARSSensitivities(self):
+        data = normalFeatureDataset(perlabel=10, nlabels=2, nfeatures=4)
+
+        # use LARS on binary problem
+        clf = LARS()
+        clf.train(data)
+
+        # now ask for the sensitivities WITHOUT having to pass the dataset
+        # again
+        sens = clf.getSensitivityAnalyzer(force_training=False)()
+
+        self.failUnless(sens.shape == (data.nfeatures,))
+
+
 def suite():
     return unittest.makeSuite(LARSTests)
 
